/******************************************************************************
 *
 * File:        region.h
 *
 * Description: This file contains the declaration of Region which is used
 * to handle information about one region, i.e. its coordinate, width, height
 * etc.
 *
 * JSON file handling is based on the example
 * https://doc.qt.io/qt-6.4/qtcore-serialization-savegame-example.html
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef REGION_H
#define REGION_H

/******************************************************************************
 * Include Files
 ******************************************************************************/
#include <QObject>
#include <QtQml/qqmlregistration.h>

namespace WindowManager {
namespace Core {

    /******************************************************************************
     * Class Declaration
     ******************************************************************************/
    /**
     * @brief Class that handles the region information in the compositor, i.e. the
     * area where one client is displayed and a list of conflicting region
     * information.
     */
    class Region : public QObject
    {
        Q_OBJECT
        Q_PROPERTY(QString id READ id CONSTANT)
        Q_PROPERTY(QString name READ name CONSTANT)
        Q_PROPERTY(double x READ x CONSTANT)
        Q_PROPERTY(double y READ y CONSTANT)
        Q_PROPERTY(double width READ width CONSTANT)
        Q_PROPERTY(double height READ height CONSTANT)
        Q_PROPERTY(QList<Region *> regions READ regions CONSTANT)
        Q_PROPERTY(QStringList conflictsWith READ conflictsWith CONSTANT)

        QML_ELEMENT

    public:
        /**
         * @brief Constructor of the class
         */
        Region(QObject *parent = nullptr);

        /**
         * @brief JSON write method that fills one QJsonObject with region data from
         * the Region object
         * @param json The JSON object to fill
         */
        void writeJson(QJsonObject &json);

        /**
         * @brief JSON read method that reads region data from one QJsonObject and fills
         * the Region object
         * @param json The JSON object to read from
         */
        int readJson(QJsonObject json, int z);

        /**
         * @brief Returns the id of the Region
         * @return QString with the id
         */
        QString id();

        /**
         * @brief Sets the id of the Region
         * @param value QString with the id
         */
        void setId(QString value);

        /**
         * @brief Returns the name of the Region
         * @return QString with the name
         */
        QString name();

        /**
         * @brief Sets the name of the Region
         * @param value QString with the name
         */
        void setName(QString value);

        /**
         * @brief Returns the X value of the topleft coordinate of the Region
         * @return double with the value
         */
        double x();

        /**
         * @brief Sets the X value of the topleft coordinate of the Region
         * @param value double with the value
         */
        void setX(double value);

        /**
         * @brief Returns the Y value of the topleft coordinate of the Region
         * @return double with the value
         */
        double y();

        /**
         * @brief Sets the Y value of the topleft coordinate of the Region
         * @param value double with the value
         */
        void setY(double value);

        /**
         * @brief Returns the Z value of the Region
         * @return int with the value
         */
        int z();

        /**
         * @brief Returns the width of the Region
         * @return double with the value
         */
        double width();

        /**
         * @brief Sets the width of the Region
         * @param value double with the value
         */
        void setWidth(double value);

        /**
         * @brief Returns the height of the Region
         * @return double with the value
         */
        double height();

        /**
         * @brief Sets the height of the Region
         * @param value double with the value
         */
        void setHeight(double value);

        /**
         * @brief Returns a list of child regions
         * @return QList<Region *> the list of regions
         */
        QList<Region *> &regions();

        /**
         * @brief Returns a list of regionIds that conflicts the region
         * @return QStringList the list of regionsIds
         */
        QStringList &conflictsWith();

        /**
         * @brief Checks if there is a region with id regionId within
         * the region. Returns true if either its the region itself,
         * or one in its nested child regions.
         * @param regionId ID of region to check for
         * @return True if region exists
         */
        bool containsRegion(QString regionId);

    private:
        void initConflictingRegions();
        void addConflictingAncestor(Region *region);
        void addConflictingSuccessor(QList<Region *> &regions);

        QString m_id;
        QString m_name;
        double m_x;
        double m_y;
        int m_z;
        double m_width;
        double m_height;
        QList<Region *> m_regions;
        QStringList m_conflictsWith;
    };

} // Core
} // WindowManager

#endif // REGION_H
